from allennlp.common.testing import AllenNlpTestCase
from allennlp.interpret.saliency_interpreters import SimpleGradient
from allennlp.models.archival import load_archive
from allennlp.predictors import Predictor

from tests import FIXTURES_ROOT


class TestSimpleGradient(AllenNlpTestCase):
    def test_simple_gradient_masked_lm(self):
        inputs = {
            "sentence": "This is a single string [MASK] about a test . Sometimes it "
            "contains coreferent parts ."
        }
        archive = load_archive(
            FIXTURES_ROOT / "lm" / "masked_language_model" / "serialization" / "model.tar.gz"
        )
        predictor = Predictor.from_archive(archive, "masked_language_model")
        interpreter = SimpleGradient(predictor)
        interpretation = interpreter.saliency_interpret_from_json(inputs)
        assert interpretation is not None
        assert "instance_1" in interpretation
        assert "grad_input_1" in interpretation["instance_1"]
        grad_input_1 = interpretation["instance_1"]["grad_input_1"]
        assert len(grad_input_1) == 16  # 16 words in input
